# frozen_string_literal: true

$LOAD_PATH.unshift("#{File.dirname(__FILE__)}/lib")
require 'landline'

app = Landline::Server.new do
  get "/hello" do
    header "content-type", "text/plain"
    "Hello World!"
  end

  get "/error" do # This error will be caught by the default server handler
    raise StandardError, "I raised an error! (and that's very sad)"
  end

  path "/handled" do
    get "/error" do # This error will be caught by the handler defined for this path
      raise StandardError, "I raised an error! (and that's very sad)"
    end

    handle do |status, backtrace: nil, error: nil|
      ([Landline::Util::HTTP_STATUS[status]] +
      (backtrace || [""])).join("\n")
    end
  end

  path "/pipelined" do
    # The pipeline executes inside the function which catches errors,
    # so it can catch errors before the handler for the current path.
    # This may be useful in order to override the way things are output.
    pipeline do |request, &output|
      output.call(request)
    rescue StandardError
      throw :finish, "Internal error caught in pipeline!"
    end

    path "/inner" do
      get "/error" do # This error will be caught by the pipeline
        raise StandardError, "I raised an error! (and that's very sad)"
      end
    end

    path "/inner_override" do
      get "/error" do # This error will be caught by the innermost handler
        raise StandardError, "I raised an error! (and that's very sad)"
      end

      handle do |status, backtrace: nil, error: nil|
        ([Landline::Util::HTTP_STATUS[status]] +
        (backtrace || [""])).join("\n")
      end
    end

    get "/error" do # This error will be caught by the pipeline
      raise StandardError, "I raised an error! (and that's very sad)"
    end

    # This handler is preceded by the pipeline, but it may handle 404 codes.
    handle do |status, backtrace: nil, error: nil|
      ([Landline::Util::HTTP_STATUS[status]] +
      (backtrace || [""])).join("\n")
    end
  end
end

run app
