# frozen_string_literal: true

module Landline
  # Rack application interface
  class App
    class << self
      # TODO: fix this mess somehow (probably impossible)
      # @!parse include Landline::DSL::PathMethods
      # @!parse include Landline::DSL::PathConstructors
      # @!parse include Landline::DSL::ProbeConstructors
      # @!parse include Landline::DSL::ProbeMethods
      # @!parse include Landline::DSL::CommonMethods

      # Duplicate used middleware for the subclassed app
      def inherited(subclass)
        super(subclass)
        @setup_chain ||= []
        subclass.middleware = @middleware.dup
        subclass.setup_chain = @setup_chain.dup
      end

      # Include a middleware in application
      # @param middleware [Class]
      def use(middleware)
        @middleware ||= []
        @middleware.append(middleware)
      end

      # Check if Server can respond to given symbol
      def respond_to_missing?(symbol, _include_private)
        Landline::ServerContext.instance_methods.include?(symbol) || super
      end

      # Store applied app manipulations
      def method_missing(symbol, *args, **params, &callback)
        if Landline::ServerContext.instance_methods.include? symbol
          @setup_chain.append([symbol, args, params, callback])
        else
          super(symbol, *args, **params, &callback)
        end
      end

      attr_accessor :middleware, :setup_chain
    end

    def initialize(*args, **opts)
      setup_chain = self.class.setup_chain
      @app = ::Landline::Server.new(*args, **opts) do
        setup_chain.each do |symbol, cargs, cparams, callback|
          send(symbol, *cargs, **cparams, &callback)
        end
      end
      self.class.middleware&.reverse_each do |cls|
        @app = cls.new(@app)
      end
    end

    # Rack ingress point.
    # @param env [Hash]
    # @return [Array(Integer,Hash,Array)]
    def call(env)
      @app.call(env)
    end
  end
end
